//------------------------------------------------------
//
//   Greg's Mod Base - Configuration
//
//------------------------------------------------------

package gcewing.blogging;

import java.io.*;
import java.util.*;
import net.minecraftforge.common.*;
import cpw.mods.fml.common.registry.*;

public class BaseConfiguration extends Configuration {

	public boolean extended = false;
	int nextVillagerID = 100;
	
	public BaseConfiguration(File file) {
		super(file);
	}
	
	public String getString(String category, String key, String defaultValue) {
		return get(category, key, defaultValue).getString();
	}

	public boolean getBoolean(String category, String key, boolean defaultValue) {
		return get(category, key, defaultValue).getBoolean(defaultValue);
	}
	
	public int getInteger(String category, String key, int defaultValue) {
		return get(category, key, defaultValue).getInt();
	}
	
	public int getVillager(String key) {
		Property prop = get("villagers", key, -1);
		int id = prop.getInt();
		if (id != -1) {
			VillagerRegistry reg = VillagerRegistry.instance();
			reg.registerVillagerId(id);
		}
		else {
			id = registerNewVillagerType();
			prop.set(id);
		}
		return id;
	}
	
	int registerNewVillagerType() {
		VillagerRegistry reg = VillagerRegistry.instance();
		Collection<Integer> inUse = VillagerRegistry.getRegisteredVillagers();
		int id;
		for (;;) {
			id = nextVillagerID++;
			if (!inUse.contains(id)) {
				reg.registerVillagerId(id);
				return id;
			}
		}
	}

	@Override
	public Property get(String category, String key, String defaultValue, String comment, Property.Type type) {
		if (!hasKey(category, key))
			extended = true;
		return super.get(category, key, defaultValue, comment, type);
	}

}
