//------------------------------------------------------------------------------------------------
//
//   Greg's Mod Base - Generic Client Proxy
//
//------------------------------------------------------------------------------------------------

package gcewing.blogging;

import java.net.*;
import java.util.*;

import net.minecraft.block.*;
import net.minecraft.client.*;
import net.minecraft.client.gui.*;
import net.minecraft.client.audio.*;
import net.minecraft.client.renderer.*;
import net.minecraft.client.renderer.tileentity.*;
import net.minecraft.entity.player.*;
import net.minecraft.inventory.*;
import net.minecraft.item.*;
import net.minecraft.network.*;
import net.minecraft.network.packet.*;
import net.minecraft.src.*;
import net.minecraft.tileentity.*;
import net.minecraft.world.*;

import net.minecraftforge.common.*;
import net.minecraftforge.client.*;
import cpw.mods.fml.common.network.*;
import cpw.mods.fml.client.*;
import cpw.mods.fml.client.registry.*;

public class BaseModClient implements IGuiHandler {

	interface IRenderType {
		public void setRenderType(int id);
	}

	public Minecraft mc;
	BaseMod base;

	Map<Integer, Class<? extends GuiScreen>> screenClasses =
		new HashMap<Integer, Class<? extends GuiScreen>>();

	public BaseModClient(BaseMod mod) {
		//System.out.printf("%s: BaseModClient()\n", this);
		base = mod;
		mc = ModLoader.getMinecraftInstance();
		registerScreens();
		registerRenderers();
		registerSounds();
		registerOther();
	}
	
	void registerOther() {}
	
	//-------------- Screen registration --------------------------------------------------------
	
	void registerScreens() {
		//
		//  Make calls to addScreen() here.
		//
		//  Screen classes registered using these methods must implement either:
		//
		//  (1) A static method create(EntityPlayer player, World world, int x, int y, int z)
		//  (2) A constructor MyScreen(EntityPlayer player, World world, int x, int y, int z)
		//
		//System.out.printf("%s: BaseModClient.registerScreens\n", this);
	}
	
	public void addScreen(Enum id, Class<? extends GuiScreen> cls) {
		addScreen(id.ordinal(), cls);
	}

	public void addScreen(int id, Class<? extends GuiScreen> cls) {
		screenClasses.put(id, cls);
	}
	
	//-------------- Renderer registration --------------------------------------------------------
	
	void registerRenderers() {
		// Make calls to addBlockRenderer(), addItemRenderer() and addTileEntityRenderer() here
	}

	void addBlockRenderer(IRenderType block, ISimpleBlockRenderingHandler renderer) {
		addBlockRenderer(renderer, block);
	}
	
	void addBlockRenderer(ISimpleBlockRenderingHandler renderer, IRenderType... blocks) {
		int renderID = RenderingRegistry.getNextAvailableRenderId();
		//renderer.renderID = renderID;
		for (IRenderType block : blocks) {
			block.setRenderType(renderID);
			RenderingRegistry.registerBlockHandler(renderID, renderer);
		}
	}
	
	void addItemRenderer(Item item, IItemRenderer renderer) {
		MinecraftForgeClient.registerItemRenderer(item.itemID, renderer);
	}
	
	void addItemRenderer(Block block, IItemRenderer renderer) {
		MinecraftForgeClient.registerItemRenderer(block.idDropped(0, null, 0), renderer);
	}
	
	void addTileEntityRenderer(Class <? extends TileEntity> teClass, TileEntitySpecialRenderer renderer) {
		ClientRegistry.bindTileEntitySpecialRenderer(teClass, renderer);
	}
	
	//-------------- Client-side guis ------------------------------------------------
	
	public static void openClientGui(GuiScreen gui) {
		FMLClientHandler.instance().getClient().displayGuiScreen(gui);
	}
	
	//-------------- Internal --------------------------------------------------------
	
	void registerSounds() {
//		try {
//			//System.out.printf("BaseModClient.registerSounds\n");
//			SoundPool pool = mc.sndManager.soundPoolSounds;
//			String namePrefix = base.modPackage.replace(".", "/") + "/";
//			String subdir = "sounds";
//			Set<String> items = base.listResources(subdir);
//			for (String item : items) {
//				//System.out.printf("BaseProxyClient.registerSounds: item = %s\n", item);
//				String soundName = namePrefix + item;
//				URL soundURL = new URL(base.resourceURL, subdir + "/" + item);
//				//System.out.printf("BaseModClient.registerSounds: name = %s url = %s\n", soundName, soundURL);
//				pool.addSound(soundName, soundURL);
//			}
//		}
//		catch (Exception e) {
//			throw new RuntimeException(e);
//		}
	}
	
	/**
	 * Returns a Container to be displayed to the user. 
	 * On the client side, this needs to return a instance of GuiScreen
	 * On the server side, this needs to return a instance of Container
	 *
	 * @param ID The Gui ID Number
	 * @param player The player viewing the Gui
	 * @param world The current world
	 * @param x X Position
	 * @param y Y Position
	 * @param z Z Position
	 * @return A GuiScreen/Container to be displayed to the user, null if none.
	 */
	
	@Override
	public Object getServerGuiElement(int id, EntityPlayer player, World world, int x, int y, int z) {
		return base.getServerGuiElement(id, player, world, x, y, z);
	}

	@Override
	public Object getClientGuiElement(int id, EntityPlayer player, World world, int x, int y, int z) {
		Class cls = screenClasses.get(id);
		if (cls != null)
			return base.createGuiElement(cls, player, world, x, y, z);
		else
			return getGuiScreen(id, player, world, x, y, z);
	}
	
	GuiScreen getGuiScreen(int id, EntityPlayer player, World world, int x, int y, int z) {
		//  Called when screen id not found in registry
		System.out.printf("%s: BaseModClient.getGuiScreen: No GuiScreen class found for gui id %d\n", 
			this, id);
		return null;
	}

}
