//------------------------------------------------------
//
//   Greg's Blogging - Blogging GUI
//
//------------------------------------------------------

package gcewing.blogging;

import java.io.*;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.util.*;
import javax.imageio.*;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import sun.misc.BASE64Encoder;

import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL12;
import org.lwjgl.opengl.GL14;

import net.minecraft.client.*;
import net.minecraft.client.gui.*;
import net.minecraft.client.multiplayer.*;
import net.minecraft.world.storage.*;
import cpw.mods.fml.client.FMLClientHandler;

import static cpw.mods.fml.common.ObfuscationReflectionHelper.*;

public class GuiBlog extends GuiScreen {

	final static int attachButtonId = 1;
	final static int sendButtonId = 2;
	final static int screenshotThumbnailHeight = 40;
	
	static String[] attachLabels = {"Add Screenshot", "Remove Screenshot"};
	//static String[] attachLabels = {"No Attachment", "Attach Screenshot"};
	//static BASE64Encoder base64 = new BASE64Encoder();
	
	public static String worldName = "Miner's Blog";
	static String title;
	static String text = "";
	static int thumbnailTexId = 0;
	static int attachState = 0;
	static File screenshotFile;
	static BufferedImage screenshotImage;
	static int screenshotThumbnailWidth;
	static List<BlogThread> threads = new ArrayList<BlogThread>();

	GuiTextMultiline inputField;

	public GuiBlog() {
	}
	
	public void initGui() {
		FontRenderer f = this.fontRenderer;
		System.out.printf("GuiBlog: Screen size = %d x %d, font height = %d\n", width, height, f.FONT_HEIGHT);
		Keyboard.enableRepeatEvents(true);
		int x = 4;
		int y = 4 + f.FONT_HEIGHT;
		inputField = new GuiTextMultiline(f, x, y, this.width - x - 4, this.height - y - 24, 2000);
		inputField.setEnableBackgroundDrawing(false);
		inputField.setFocused(true);
		inputField.setCanLoseFocus(false);
		inputField.setText(text);
		buttonList.add(new GuiButton(attachButtonId, 0, height - 20, 120, 20, attachLabels[attachState]));
		buttonList.add(new GuiButton(sendButtonId, width - 120, height - 20, 120, 20, "Send"));
		updateTitle();
	}
	
	void updateTitle() {
		// Unfortunately, WorldInfo.getWorldName() doesn't return anything useful on
		// the client side, so we resort to various subterfuges to find a reasonable
		// default for the blog title.
		ServerData server = (ServerData)getPrivateValue(Minecraft.class, mc, "field_71422_O"); //currentServerData
		WorldInfo info = mc.theWorld.getWorldInfo();
		if (server != null) {
			System.out.printf("GuiBlog: Server = %s\n", server.serverName);
			setWorldName(server.serverMOTD);
		}
		else {
			//System.out.printf("GuiBlog: No server data\n");
		}
		long date = info.getWorldTotalTime() / 24000;
		title = String.format("%s, Minedate %04d.%d", worldName, date / 10, date % 10);
	}
	
	public static void setWorldName(String newName) {
		if (!worldName.equals(newName)) {
			worldName = newName;
			WordpressThread.reset();
		}
	}
	
	public static enum SendState {
		Idle, Sending, Sent, Error
	}
	
	public static SendState sendingState() {
		if (threads.isEmpty())
			return SendState.Idle;
		else if (allThreadsComplete())
			return SendState.Sent;
		else if (anyThreadError())
			return SendState.Error;
		else
			return SendState.Sending;
	}
	
	static boolean allThreadsComplete() {
		for (BlogThread thread : threads)
			if (!thread.complete)
				return false;
		return true;
	}
	
	static boolean anyThreadError() {
		for (BlogThread thread : threads)
			if (thread.hasFailed())
				return true;
		return false;
	}

	public static List<String> getSendErrors() {
		List<String> result = new ArrayList<String>();
		for (BlogThread thread : threads)
			if (thread.error != null)
				result.add(thread.error.getMessage());
		return result;
	}
	
	public static void reset() {
		title = null;
		text = "";
		attachState = 0;
		screenshotFile = null;
		threads.clear();
	}
	
	File findMostRecentScreenshot() {
		File mcDir = FMLClientHandler.instance().getClient().mcDataDir; //getMinecraftDir();
		File ssDir = new File(mcDir, "screenshots");
		System.out.printf("GuiBlog: Looking for screenshots in: %s\n", ssDir.getPath());
		String[] names = ssDir.list();
		if (names != null && names.length > 0) {
			Arrays.sort(names);
			for (int i = 0; i < names.length; i++)
				System.out.printf("[%d] %s\n", i, names[i]);
			return new File(ssDir, names[names.length - 1]);
		}
		return null;
	}

	public void onGuiClosed() {
		text = inputField.getText();
		Keyboard.enableRepeatEvents(false);
	}

	public void updateScreen() {
		this.inputField.updateCursorCounter();
	}

	@Override
	protected void actionPerformed(GuiButton button) {
		switch (button.id) {
			case attachButtonId:
				attachState = 1 - attachState;
				button.displayString = attachLabels[attachState];
				if (attachState == 1) {
					screenshotFile = findMostRecentScreenshot();
					//System.out.printf("GuiBlog: Screenshot = %s\n", screenshotFile);
					updateScreenshot();
				}
				break;
			case sendButtonId:
				send();
				break;
		}
	}

	protected void keyTyped(char c, int key) {
		//System.out.printf("GuiBlog.keyTyped: %d %d\n", (int)c, key);
		if (key == Keyboard.KEY_ESCAPE)
			mc.displayGuiScreen(null);
		//else if (c == 19) // Ctrl-S
		//	send();
		else
			inputField.textboxKeyTyped(c, key);
	}
	
	protected void mouseClicked(int par1, int par2, int par3) {
		inputField.mouseClicked(par1, par2, par3);
		super.mouseClicked(par1, par2, par3);
	}
	
	void send() {
		mc.displayGuiScreen(null);
		sendBlog();
	}
	
	public static void sendBlog() {
		//String text = this.inputField.getText();
		//System.out.printf("GuiBlog: Text entered: %s\n", text.replace('\n', '|'));
		byte[] image = null;
		String html = "<html><body>" +  htmlEscape(text);
		if (attachState > 0) {
			image = convertScreenshotToJpg();
			html += "<br><br><img src=\"<IMGSRC>\">";
		}
		html += "</body></html>";
		if (SMTP.isEnabled())
			sendUsingMail(html, image);
		if (Wordpress.isEnabled())
			sendUsingWordpress(html, image);
		if (threads.isEmpty())
			Chat.print("No blog sending methods configured");
	}
	
	public static void resendBlog() {
		for (BlogThread thread : threads)
			if (thread.hasFailed())
				thread.start();
	}
	
	static void sendUsingMail(String html, byte[] image) {
		startThread(new EmailThread(title, html, image));
	}
	
	static void sendUsingWordpress(String html, byte[] image) {
		startThread(new WordpressThread(title, html, image));
	}
	
	static void startThread(BlogThread thread) {
		threads.add(thread);
		thread.start();
	}
	
	static String htmlEscape(String s) {
		return s.replace("&", "&amp;").replace("<", "&lt;").replace("\n", "<br>");
	}
	
	static byte[] convertScreenshotToJpg() {
		if (screenshotImage != null) {
			try {
				int w = screenshotImage.getWidth() * 3 / 4;
				int h = screenshotImage.getHeight() * 3 / 4;
				BufferedImage img = scaleImage(screenshotImage, w, h, BufferedImage.TYPE_INT_RGB);
				//BufferedImage img = screenshotImage;
				//Image img = screenshotImage.getScaledInstance(w, h, Image.SCALE_SMOOTH);
				ByteArrayOutputStream buf = new ByteArrayOutputStream();
				ImageIO.write(img, "jpg", buf);
				//ImageIO.write(img, "png", buf);
				return buf.toByteArray();
			}
			catch (IOException e) {
				String mess = "Error converting screenshot to jpg: " + e;
				System.out.printf("GregsBlogging: %s\n", mess);
				Chat.print(mess);
			}
		}
		return null;
	}
	
	void updateScreenshot() {
		screenshotImage = null;
		if (screenshotFile != null) {
			try {
				screenshotImage = ImageIO.read(screenshotFile);
				int h = screenshotThumbnailHeight;
				int w = (h * screenshotImage.getWidth()) / screenshotImage.getHeight();
				BufferedImage buf = scaleImage(screenshotImage, w, h, BufferedImage.TYPE_INT_ARGB);
				IntBuffer data = ByteBuffer.allocateDirect(4 * w * h).asIntBuffer();
				data.put(buf.getRGB(0, 0, w, h, null, 0, w));
				bindThumbnailTexture();
				data.position(0);
				GL11.glTexSubImage2D(GL11.GL_TEXTURE_2D, 0, 0, 0, w, h,
					GL12.GL_BGRA, GL12.GL_UNSIGNED_INT_8_8_8_8, data);
				screenshotThumbnailWidth = w;
			}
			catch (IOException e) {
				String mess = "Error reading screenshot file " + screenshotFile + ": " + e;
				System.out.printf("GregsBlogging: %s\n", mess);
				Chat.print(mess);
			}
		}
	}
	
	static BufferedImage scaleImage(Image src, int w, int h, int type) {
		Image img = src.getScaledInstance(w, h, Image.SCALE_SMOOTH);
		BufferedImage buf = new BufferedImage(w, h, type);
		Graphics2D g = buf.createGraphics();
		g.drawImage(img, null, null);
		return buf;
	}
	
	void bindThumbnailTexture() {
		int tex = thumbnailTexId;
		if (tex == 0) {
			tex = GL11.glGenTextures();
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, tex);
			GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL11.GL_REPEAT);
			GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL11.GL_REPEAT);
			GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST);
			GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST);
			GL11.glPixelStorei(GL11.GL_UNPACK_ALIGNMENT, 1);			
			GL11.glTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_RGBA, 256, 256, 0,
				GL12.GL_BGRA, GL12.GL_UNSIGNED_INT_8_8_8_8, (IntBuffer)null);
			thumbnailTexId = tex;
		}
		else
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, tex);
	}
	
	public void drawScreen(int par1, int par2, float par3) {
		FontRenderer f = fontRenderer;
		drawString(f, title, 4, 4, 0xffffff00);
		drawRect(0, 0, width, height, Integer.MIN_VALUE);
		inputField.drawTextBox();
		drawScreenshotThumbnail();
		super.drawScreen(par1, par2, par3);
	}

	void drawScreenshotThumbnail() {
		if (screenshotFile != null && attachState > 0) {
			int w = screenshotThumbnailWidth;
			int h = screenshotThumbnailHeight;
			int x = 124;
			int y = height - h - 2;
			//System.out.printf("GuiBlog: Drawing thumbnail at (%d, %d) size %d x %d\n", x, y, w, h);
			drawRectFrame(x, y, x+w, y+h, 2, 0xff000000);
			bindThumbnailTexture();
			GL11.glColor4f(1, 1, 1, 1);
			drawTexturedModalRect(x, y, 0, 0, w, h);
		}
	}
	
	void drawRectFrame(int left, int top, int right, int bottom, int thick, int color) {
		drawRect(left-thick, top-thick, left, bottom+thick, color);
		drawRect(right, top-thick, right+thick, bottom+thick, color);
		drawRect(left, top-thick, right, top, color);
		drawRect(left, bottom, right, bottom+thick, color);
	}
	
}
