//----------------------------------------------------------------------
//
//   Greg's Blogging - GUI for reporting in-progress or error status
//
//----------------------------------------------------------------------

package gcewing.blogging;

import java.util.*;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;

import net.minecraft.client.gui.*;
import cpw.mods.fml.client.FMLClientHandler;

import gcewing.blogging.GuiBlog.SendState;

public class GuiStatus extends GuiScreen {

	final static int newButtonId = 1;
	final static int retryButtonId = 2;
	final static int backButtonId = 3;
	final static int editButtonId = 4;
	
	GuiButton newButton, retryButton, backButton, editButton;
	String statusText;
	List<String> errorText = new ArrayList<String>();
	
	public GuiStatus() {
	}
	
	@Override
	public void initGui() {
		int w = 120;
		int x = (width - w) / 2;
		int y = height / 2;
		retryButton = new GuiButton(retryButtonId, x, y +  0, w, 20, "Send Again");
		newButton =   new GuiButton(newButtonId,   x, y + 25, w, 20, "New Entry");
		editButton =  new GuiButton(editButtonId,  x, y + 50, w, 20, "Edit Entry");
		backButton =  new GuiButton(backButtonId,  x, y + 75, w, 20, "Return to Game");
		buttonList.add(newButton);
		buttonList.add(retryButton);
		buttonList.add(backButton);
		buttonList.add(editButton);
		updateScreen();
	}
	
	@Override
	public void updateScreen() {
		switch (GuiBlog.sendingState()) {
			case Idle:
			case Sent:
				newEntry();
				break;
			case Sending:
				statusText = "Still sending last blog entry, please wait";
				errorText.clear();
				newButton.drawButton = false;
				editButton.drawButton = false;
				retryButton.drawButton = false;
				break;
			case Error:
				statusText = "Sending the last blog entry failed:";
				errorText = GuiBlog.getSendErrors();
				newButton.drawButton = true;
				editButton.drawButton = true;
				retryButton.drawButton = true;
				break;
		}
	}
	
	@Override
	protected void keyTyped(char c, int key) {
		System.out.printf("GuiBlog.keyTyped: %d %d\n", (int)c, key);
		if (key == Keyboard.KEY_ESCAPE)
			mc.displayGuiScreen(null);
	}

	@Override
	protected void actionPerformed(GuiButton button) {
		switch (button.id) {
			case newButtonId:
				newEntry();
				break;
			case editButtonId:
				editEntry();
				break;
			case retryButtonId:
				retry();
				break;
			case backButtonId:
				mc.displayGuiScreen(null);
				break;
		}
	}
	
	void newEntry() {
		GuiBlog.reset();
		editEntry();
	}
	
	void editEntry() {
		GregsBloggingClient.openGuiBlog();
	}
	
	void retry() {
		mc.displayGuiScreen(null);
		GuiBlog.resendBlog();
	}

	@Override	
	public void drawScreen(int par1, int par2, float par3) {
		drawRect(0, 0, width, height, Integer.MIN_VALUE);
		drawMessages();
		super.drawScreen(par1, par2, par3);
	}
	
	void drawMessages() {
		FontRenderer f = fontRenderer;
		int fh = f.FONT_HEIGHT + 5;
		int x = width / 2;
		//int y = height / 2 - 2 * fh;
		int y = fh;
		drawCenteredString(f, statusText, x, y, 0xffffffff);
		for (String s : errorText) {
			y += fh;
			drawCenteredString(f, s, x, y, 0xffffffff);
		}
	}
	
}
