//------------------------------------------------------
//
//   Greg's Blogging - Key binding list
//
//------------------------------------------------------

package gcewing.blogging;

import java.util.ArrayList;
import net.minecraft.client.settings.KeyBinding;
import cpw.mods.fml.common.registry.LanguageRegistry;

public class KeyBindingList {

	ArrayList <KeyBinding> bindings;
	ArrayList <Boolean> repeats;
	
	public KeyBindingList() {
		bindings = new ArrayList <KeyBinding> ();
		repeats = new ArrayList <Boolean> ();
	}
	
	public void add(KeyBinding binding) {
		add(binding, false);
	}

	public void add(KeyBinding binding, boolean repeat) {
		bindings.add(binding);
		repeats.add(repeat);
	}
	
	public void bind(String name, String title, int keyCode, IKeyHandler handler) {
		String bindingName = "key.gcewing." + name;
		add(new KeyHandlerBinding(bindingName, keyCode, handler));
		LanguageRegistry.instance().addStringLocalization(bindingName, title);
	}
	
	public KeyBinding[] getBindings() {
		return bindings.toArray(new KeyBinding[bindings.size()]);
	}
	
	public boolean[] getRepeats() {
		int n = bindings.size();
		boolean[] result = new boolean[n];
		for (int i = 0; i < n; i++)
			result[i] = repeats.get(i).booleanValue();
		return result;
	}

}

//--------------------------------------------------------------------------------------------------

class KeyHandlerBinding extends KeyBinding {

	public IKeyHandler handler;

	KeyHandlerBinding(String name, int keyCode, IKeyHandler handler) {
		super(name, keyCode);
		this.handler = handler;
	}

}
