//------------------------------------------------------------------------------
//
//   Greg's Blogging Mod - Wordpress protocol
//
//------------------------------------------------------------------------------

package gcewing.blogging;

import java.util.*;
import gcewing.blogging.web.*;

public class Wordpress {

	public static boolean debug = false;

	public static Site site;
	
	public static void configure(BaseConfiguration config) {
		try {
			boolean enabled = config.getBoolean("wordpress", "enabled", true);
			debug = config.getBoolean("wordpress", "debug", debug);
			Site site = new Site();
			site.host = config.getString("wordpress", "host", "wordpress.com");
			site.username = config.getString("wordpress", "username", "");
			site.password = config.getString("wordpress", "password", "");
			site.defaultBlogName = config.getString("wordpress", "blogName", "");
			if (enabled) {
				//site.loadBlogs();
				//site.setDefaultBlog(blogName);
				Wordpress.site = site;
			}
		}
		catch (EnvironmentError e) {
			GregsBlogging.logError("Error configuring Wordpress: %s", e.getMessage());
		}
	}
	
	public static boolean isEnabled() {
		return site != null && !site.username.equals("");
	}

	public static class Site {
		public String host = "wordpress.com";
		public String username;
		public String password;
		public String defaultBlogName;
		public Map<String, Blog> blogs = new HashMap<String, Blog>();
		public Blog defaultBlog;
		
		String endpoint() {
			return "https://" + host + "/xmlrpc.php";
		}
		
		List<Blog> getUsersBlogs() {
			if (debug)
				System.out.printf("GregsBlogging: Getting list of blogs from %s\n", host);
			Array resp = XMLRPC.callMethod(endpoint(), "wp.getUsersBlogs", username, password);
			Array items = resp.getArray(0);
			int n = items.size();
			List<Blog> blogs = new ArrayList<Blog>();
			for (int i = 0; i < n; i++) {
				Blog blog = new Blog(this, items.getStruct(0));
				if (debug)
					System.out.printf("GregsBlogging: Found blog \"%s\" id %s\n", blog.blogName, blog.blogid);
				blogs.add(blog);
			}
			return blogs;
		}
		
		void loadBlogs() {
			for (Blog blog : getUsersBlogs())
				blogs.put(blog.blogName, blog);
			setDefaultBlog(defaultBlogName);
		}
		
		void setDefaultBlog(String name) {
			if (!name.equals("")) {
				defaultBlog = getBlog(name);
				if (defaultBlog == null)
					GregsBlogging.logError("Blog \"%s\" not found at %s", name, host);
			}
			else {
				Collection<Blog> values = blogs.values();
				int n = values.size();
				if (n == 0)
					GregsBlogging.logError("No blogs found at %s", host);
				else if (n > 1)
					GregsBlogging.logError("More than one blog found at %s and no blogName in config", host);
				else
					defaultBlog = values.iterator().next();
			}
		}
		
		Blog getBlog(String name) {
			Blog result = blogs.get(name);
			if (result == null)
				System.out.printf("GregsBlogging: ERROR: Blog \"%s\" not found at %s\n",
					name, host);
			return result;
		}
		
	}
	
	public static class Blog {
		Site site;
		String blogName;
		String xmlrpc;
		String blogid;
		String url;
		
		public Blog(Site site, Struct s) {
			this.site = site;
			blogName = s.getString("blogName");
			xmlrpc = s.getString("xmlrpc");
			blogid = s.getString("blogid");
			url = s.getString("url");
		}
		
		public void dump() {
			System.out.printf("blogName: \"%s\"\n", blogName);
			System.out.printf("xmlrpc: \"%s\"\n", xmlrpc);
			System.out.printf("blogid: \"%s\"\n", blogid);
			System.out.printf("url: \"%s\"\n", url);
		}
	
		String newPost(String title, String body) {
			Struct content = new Struct();
			content.put("post_title", title);
			content.put("post_content", body);
			content.put("post_status", "publish");
			Array resp = XMLRPC.callMethod(xmlrpc, "wp.newPost",
				blogid, site.username, site.password, content);
			return resp.getString(0);
		}
		
		boolean editPost(String postID, String body) {
			Struct content = new Struct();
			content.put("post_content", body);
			Array resp = XMLRPC.callMethod(xmlrpc, "wp.editPost",
				blogid, site.username, site.password, postID, content);
			return resp.getBoolean(0);
		}
		
		String uploadFile(String title, String type, byte[] data) {
			Struct params = new Struct();
			params.put("name", title);
			params.put("type", type);
			params.put("bits", data);
			Array resp = XMLRPC.callMethod(xmlrpc, "wp.uploadFile",
				blogid, site.username, site.password, params);
			Struct ret = resp.getStruct(0);
			return ret.getString("url");
		}

	}

}
	